
/* A program that modifies programs so that they may print themselves */
/* Shaun Flisakowski */

#include  <stdio.h>
#include  <string.h>

#define   SP_EXTRA_LINES    60

int    num_lines = 0;
int    max_args  = 1;

#define    BUF_LEN    1000

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
void analyze_file(FILE *fp)
{
    int  j, len;
    int  cur_args = 1;
    char buff[BUF_LEN+1];

    buff[BUF_LEN] = '\0';
    while (fgets(buff,BUF_LEN,fp) != NULL) {
        len = strlen(buff);
        if (buff[len-1] == '\n') {
            num_lines++;
            if (cur_args > max_args)
                max_args = cur_args;
            cur_args = 1;
        }
        for (j=0; j<len; j++) {
            switch (buff[j]) {

            case '"':
            case '\\':
            case '@':
                cur_args++;
                break;

            default:
                break;
            }
        }
    }

      /* Return file pointer to beginning of file */
    fseek(fp,0L,SEEK_SET);
}

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
void write_head(FILE *fp)
{
    FILE *cd_fp;
    char  buff[BUF_LEN+1];

    if ((cd_fp = fopen("code/code1", "r")) == NULL) {
        fprintf(stderr, "Error: could not find file code/code1\n");
        return;
    }

    buff[BUF_LEN] = '\0';

    while (fgets(buff,BUF_LEN,cd_fp) != NULL)
        fputs(buff,fp);

    fprintf(fp,"#define    PRG_LEN   %d\n", num_lines+SP_EXTRA_LINES);

    fclose(cd_fp);    

    if ((cd_fp = fopen("code/code2", "r")) == NULL) {
        fprintf(stderr, "Error: could not find file code/code2\n");
        return;
    }

    while (fgets(buff,BUF_LEN,cd_fp) != NULL)
        fputs(buff,fp);

    fprintf(fp,"    \"#define    PRG_LEN   %d\",\n", num_lines+SP_EXTRA_LINES);

    fclose(cd_fp);

    if ((cd_fp = fopen("code/code3", "r")) == NULL) {
        fprintf(stderr, "Error: could not find file code/code3\n");
        return;
    }

    while (fgets(buff,BUF_LEN,cd_fp) != NULL)
        fputs(buff,fp);

    fclose(cd_fp);
}

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
void write_code_array( FILE *ifp, FILE *ofp)
{
    int   j, len, init=1;
    char  buff[BUF_LEN+1];
    FILE *cd_fp;

    buff[BUF_LEN] = '\0';
    while (fgets(buff,BUF_LEN,ifp) != NULL) {

        len = strlen(buff);
        if (buff[len-1] == '\n') {
                buff[--len] = '\0';
            if (!init)
                fputs("\",\n",ofp);
            fputs("    \"",ofp);
            init = 0;
        }

        for (j=0; j<len; j++) {
            switch (buff[j]) {

            case '"':
                fputs("@'",ofp);
                break;

            case '\\':
                fputs("@/",ofp);
                break;

            case '@':
                fputs("@@",ofp);
                break;

            default:
                fputc(buff[j],ofp);
                break;
            }
        }
    }

    if (!init)
        fputs("\",\n",ofp);

    fputs("  };\n\n",ofp);

    if ((cd_fp = fopen("code/code4", "r")) == NULL) {
        fprintf(stderr, "Error: could not find file code/code4\n");
        return;
    }

    while (fgets(buff,BUF_LEN,cd_fp) != NULL)
        fputs(buff,ofp);

    fclose(cd_fp);

      /* Return file pointer to beginning of file */
    fseek(ifp,0L,SEEK_SET);
}

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
void dup_code(FILE *ifp, FILE *ofp)
{
    char buff[BUF_LEN+1];

    while (fgets(buff,BUF_LEN,ifp) != NULL)
        fputs(buff,ofp);

      /* Return file pointer to beginning of file */
    fseek(ifp,0L,SEEK_SET);
}

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
int main(int ac, char **av)
{
    FILE  *in, *out;
    char  *ofile = "self_print_out.c";

    if (ac < 2) {
        fprintf(stderr,"\nUsage: %s <file to convert>\n",av[0]);
        return(0);
    }

    if ((in = fopen(av[1],"r")) == NULL) {
        fprintf(stderr,"\nError: file '%s' not found.\n",av[1]);
        return(1);
    }

    if (av[2])
        ofile = av[2];

    if ((out = fopen(ofile,"w")) == NULL) {
        fprintf(stderr,"\nError: could not open file '%s' for writing.\n",
            ofile);
        return(1);
    }

    analyze_file(in);

    fprintf(stdout, "File %s has %d lines and requires %d arguments.\n",
            av[1], num_lines, max_args );

    write_head(out);

    write_code_array(in,out);

    dup_code(in,out);

    fclose(in);
    fclose(out);

    return(0);
}

/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */
